@@ -429,7 +429,7 @@ static void fp_random(element_ptr a)
     mpz_clear(z);
 }
 
-static void fp_from_hash(element_ptr a, int len, void *data)
+static void fp_from_hash(element_ptr a, void *data, int len)
 {
     mpz_t z;
 
@@ -466,22 +466,6 @@ static int fp_sgn_odd(element_ptr a)
     }
 }
 
-static int fp_sgn_even(element_ptr a)
-{
-    dataptr ad = a->data;
-    if (!ad->flag) return 0;
-    else {
-	mpz_t z;
-	mpz_init(z);
-	int res;
-	fp_to_mpz(z, a);
-	mpz_add(z, z, z);
-	res = mpz_cmp(z, a->field->order);
-	mpz_clear(z);
-	return res;
-    }
-}
-
 static int fp_is_sqr(element_ptr a)
 {
     dataptr ad = a->data;
@@ -539,7 +523,7 @@ static void fp_field_clear(field_t f)
     free(p);
 }
 
-void fp_print_info(FILE *out, field_ptr f)
+void fp_out_info(FILE *out, field_ptr f)
 {
     element_fprintf(out, "F_p: Montgomery representation,\nmodulus = %Zd\n",
 	    f->order);
@@ -563,7 +547,7 @@ void field_init_mont_fp(field_ptr f, mpz_t prime)
     f->halve = fp_halve;
     f->pow_mpz = fp_pow_mpz;
     f->neg = fp_neg;
-    f->sign = mpz_odd_p(prime) ? fp_sgn_odd : fp_sgn_even;
+    f->sign = fp_sgn_odd;
     f->cmp = fp_cmp;
     f->invert = fp_invert;
     f->random = fp_random;
@@ -578,7 +562,7 @@ void field_init_mont_fp(field_ptr f, mpz_t prime)
     f->to_bytes = fp_to_bytes;
     f->from_bytes = fp_from_bytes;
     f->to_mpz = fp_to_mpz;
-    f->print_info = fp_print_info;
+    f->out_info = fp_out_info;
 
     p = f->data = malloc(sizeof(fp_field_data_t));
     p->limbs = mpz_size(prime);
