@@ -301,13 +301,13 @@ static void polymod_random(element_ptr e)
     }
 }
 
-static void polymod_from_hash(element_ptr e, int len, void *data)
+static void polymod_from_hash(element_ptr e, void *data, int len)
 {
     //TODO: improve this
     element_t *coeff = e->data;
     int i, n = polymod_field_degree(e->field);
     for (i=0; i<n; i++) {
-	element_from_hash(coeff[i], len, data);
+	element_from_hash(coeff[i], data, len);
     }
 }
 
@@ -612,6 +612,13 @@ static void poly_to_mpz(mpz_t z, element_ptr e)
     }
 }
 
+void poly_out_info(FILE *str, field_ptr f)
+{
+    poly_field_data_ptr p = f->data;
+    fprintf(str, "Polynomial ring. Base field:\n");
+    field_out_info(str, p->field);
+}
+
 void field_init_poly(field_ptr f, field_ptr base_field)
 {
     poly_field_data_ptr p;
@@ -642,6 +649,7 @@ void field_init_poly(field_ptr f, field_ptr base_field)
     f->mul_mpz = poly_mul_mpz;
     f->mul_si = poly_mul_si;
     f->cmp = poly_cmp;
+    f->out_info = poly_out_info;
 
     f->to_bytes = poly_to_bytes;
     f->from_bytes = poly_from_bytes;
@@ -1414,6 +1422,14 @@ static void compute_x_powers(field_ptr field, element_ptr poly)
     element_clear(p0);
 }
 
+void polymod_out_info(FILE *str, field_ptr f)
+{
+    polymod_field_data_ptr p = f->data;
+    element_fprintf(str, "Field extension. Min poly = %B\n", p->poly);
+    fprintf(str, "Base field:\n");
+    field_out_info(str, p->field);
+}
+
 void field_init_polymod(field_ptr f, element_ptr poly)
     //assumes poly is monic
 {
@@ -1471,6 +1487,8 @@ void field_init_polymod(field_ptr f, element_ptr poly)
     f->sqrt = polymod_sqrt;
     f->to_bytes = polymod_to_bytes;
     f->from_bytes = polymod_from_bytes;
+    f->out_info = polymod_out_info;
+
     if (pdp->field->fixed_length_in_bytes < 0) {
 	f->fixed_length_in_bytes = -1;
 	f->length_in_bytes = polymod_length_in_bytes;
@@ -1483,10 +1501,11 @@ void field_init_polymod(field_ptr f, element_ptr poly)
     compute_x_powers(f, poly);
 }
 
-void trial_divide(darray_ptr factor, darray_ptr mult, mpz_t n)
+void trial_divide(darray_ptr factor, darray_ptr mult, mpz_t n, mpz_ptr limit)
 {
     mpz_t p, m;
-    mpz_ptr fac, mul;
+    mpz_ptr fac;
+    unsigned int mul;
 
     mpz_init(p);
     mpz_init(m);
@@ -1497,18 +1516,20 @@ void trial_divide(darray_ptr factor, darray_ptr mult, mpz_t n)
 	if (mpz_probab_prime_p(m, 10)) {
 	    mpz_set(p, m);
 	}
+	if (limit && mpz_cmp(p, limit) > 0) {
+	    mpz_set(p, m);
+	}
 	if (mpz_divisible_p(m, p)) {
 	    fac = malloc(sizeof(mpz_t));
-	    mul = malloc(sizeof(mpz_t));
+	    mul = 0;
 	    mpz_init(fac);
-	    mpz_init(mul);
 	    mpz_set(fac, p);
 	    darray_append(factor, fac);
-	    darray_append(mult, mul);
 	    do {
 		mpz_divexact(m, m, p);
-		mpz_add_ui(mul, mul, 1);
+		mul++;
 	    } while (mpz_divisible_p(m, p));
+	    darray_append(mult, (void *) mul);
 	}
 	mpz_nextprime(p, p);
     }
@@ -1612,11 +1633,10 @@ int poly_is_irred(element_ptr f)
 
     mpz_set_ui(deg, poly_degree(f));
 
-    trial_divide(fac, mul, deg);
+    trial_divide(fac, mul, deg, NULL);
     res = poly_is_irred_degfac(f, fac);
 
     darray_forall(fac, clear);
-    darray_forall(mul, clear);
 
     darray_clear(fac);
     darray_clear(mul);
